// Copyright (c) 2017, teja. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:async';
import 'package:jaguar_reflect/jaguar_reflect.dart';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_oauth/jaguar_oauth.dart';
import 'package:oauth2/oauth2.dart' as oauth2;
import 'package:logging/logging.dart';

const JaguarOauth2Config facebook = const JaguarOauth2Config();

@RouteGroup(path: '/auth')
class AuthRoutes {
  @Get(path: '/fb/authreq')
  @Wrap(const [#oAuth2Req])
  void fbAuthReq() {}

  @Get(path: '/fb/authorized', mimeType: 'application/json')
  @Wrap(const [#oAuth2Authorized])
  Future<String> fbAuthorized(
      @Input(OAuth2Authorized) oauth2.Client client) async {
    final resp = await client.get('https://graph.facebook.com/v2.8/me');
    return resp.body;
  }

  WrapOAuth2Req oAuth2Req() => new WrapOAuth2Req(facebook());

  WrapOAuth2Authorized oAuth2Authorized() => new WrapOAuth2Authorized(facebook());

  JaguarOauth2Config facebook() => new JaguarOauth2Config(
      key: Settings.getString('facebook_oauth_key'),
      secret: Settings.getString('facebook_oauth_secret'),
      authorizationEndpoint: 'https://www.facebook.com/dialog/oauth',
      tokenEndpoint: 'https://graph.facebook.com/v2.8/oauth/access_token',
      callback: Settings.getString('baseurl') + '/api/auth/fb/authorized',
      scopes: ['email']);
}

@Api(path: '/api')
class MyApi {
  @Group()
  final AuthRoutes auth = new AuthRoutes();
}

main(List<String> args) async {
  Settings.parse(args);
  Jaguar server = new Jaguar();
  server.addApi(new JaguarReflected(new MyApi()));

  server.log.onRecord.listen((LogRecord rec) {
    print('${rec.level.name}: ${rec.time}: ${rec.message}');
  });

  await server.serve();
}
