import 'dart:async';
import 'package:jaguar_reflect/jaguar_reflect.dart';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_oauth/jaguar_oauth.dart';
import 'package:oauth2/oauth2.dart' as oauth2;
import 'package:logging/logging.dart';
import 'package:jaguar_facebook_client/jaguar_facebook_client.dart' as fb;

class User {
  /// ID for the user in the database
  String id;

  /// User name
  String name;

  /// User's email
  String email;

  String dateOfBirth;

  String bio;

  /// Token to access user's facebook account
  String fbToken;

  /// Token to refresh user's facebook account token [fbToken]
  //TODO needed? String fbRefreshToken;

  /// Facebook ID of the user
  String fbId;
}

const JaguarOauth2Config facebook = const JaguarOauth2Config();

@RouteGroup(path: '/auth')
class AuthRoutes {
  @Get(path: '/fb/authreq')
  @Wrap(const [#oAuth2Req])
  void fbAuthReq() {}

  @Get(path: '/fb/authorized', mimeType: 'application/json')
  @Wrap(const [#oAuth2Authorized])
  Future<String> fbAuthorized(
      @Input(OAuth2Authorized) oauth2.Client client) async {
    final graph = new fb.GraphApi(client);
    final fields = new fb.UserFieldSelector()
      ..addBirthday()
      ..addAbout()
      ..addEmail()
      ..addName();
    final resp = await graph.getMe(fields: fields);
    return resp.map.toString();
  }

  WrapOAuth2Req oAuth2Req() => new WrapOAuth2Req(facebook());

  WrapOAuth2Authorized oAuth2Authorized() => new WrapOAuth2Authorized(facebook());

  JaguarOauth2Config facebook() => new JaguarOauth2Config(
      key: Settings.getString('facebook_oauth_key'),
      secret: Settings.getString('facebook_oauth_secret'),
      authorizationEndpoint: 'https://www.facebook.com/dialog/oauth',
      tokenEndpoint: 'https://graph.facebook.com/v2.8/oauth/access_token',
      callback: Settings.getString('baseurl') + '/api/auth/fb/authorized',
      scopes: [fb.Scope.email, fb.Scope.userAboutMe, fb.Scope.publicProfile]);
}

@Api(path: '/api')
class MyApi {
  @Group()
  final AuthRoutes auth = new AuthRoutes();
}

main(List<String> args) async {
  Settings.parse(args);
  Jaguar conf = new Jaguar();
  conf.addApi(new JaguarReflected(new MyApi()));

  conf.log.onRecord.listen((LogRecord rec) {
    print('${rec.level.name}: ${rec.time}: ${rec.message}');
  });

  await conf.serve();
}
