// Copyright (c) 2017, teja. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library jaguar_oauth2;

import 'dart:io';
import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_auth/jaguar_auth.dart';
import 'package:oauth2/oauth2.dart' as oauth2;

part 'options.dart';

/// Sends authorization request to remote server by redirecting the browser to
/// remote server's authorization request page
class WrapOAuth2Req extends RouteWrapper<OAuth2Req> {
  final JaguarOauth2Config config;

  final String id = null;

  const WrapOAuth2Req(this.config);

  OAuth2Req createInterceptor() => new OAuth2Req(config);
}

/// Sends authorization request to remote server by redirecting the browser to
/// remote server's authorization request page
class OAuth2Req extends Interceptor {
  final JaguarOauth2Config config;

  OAuth2Req(this.config);

  Future<Response<Uri>> post(@InputRouteResponse() Response incoming) async {
    final Response<Uri> ret = new Response.cloneExceptValue(incoming);
    ret.statusCode = HttpStatus.MOVED_TEMPORARILY;
    ret.value = makeGranter(config)
        .getAuthorizationUrl(Uri.parse(config.callback), scopes: config.scopes);
    return ret;
  }
}

/// Handles OAuth2 authorized response. Ideally the route this interceptor
/// wraps must be used as callback in authorization request to remote server
class WrapOAuth2Authorized extends RouteWrapper<OAuth2Authorized> {
  final JaguarOauth2Config config;

  final String id = null;

  const WrapOAuth2Authorized(this.config);

  OAuth2Authorized createInterceptor() => new OAuth2Authorized(config);
}

/// Handles OAuth2 authorized response. Ideally the route this interceptor
/// wraps must be used as callback in authorization request to remote server
class OAuth2Authorized extends Interceptor {
  final JaguarOauth2Config config;

  OAuth2Authorized(this.config);

  Future<oauth2.Client> pre(@InputQueryParams() Map queryParams) async {
    final granter = makeGranter(config);
    granter.getAuthorizationUrl(Uri.parse(config.callback),
        scopes: config.scopes);
    return await granter.handleAuthorizationResponse(queryParams);
  }
}

oauth2.AuthorizationCodeGrant makeGranter(JaguarOauth2Config config) =>
    new oauth2.AuthorizationCodeGrant(
        config.key,
        Uri.parse(config.authorizationEndpoint),
        Uri.parse(config.tokenEndpoint),
        secret: config.secret,
        basicAuth: false);
